==> openGion_dev_5.4.3.7.zip: opengion/uap/webapps/gf/src/org/opengion/plugin/column/Renderer_AUTOAREA.java <==
/*
 * Copyright (c) 2009 The openGion Project.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.opengion.plugin.column;

import java.util.StringTokenizer;

import org.opengion.fukurou.util.StringUtil;
import org.opengion.fukurou.util.TagBuffer;
import org.opengion.hayabusa.common.HybsSystem;
import org.opengion.hayabusa.db.AbstractRenderer;
import org.opengion.hayabusa.db.CellRenderer;
import org.opengion.hayabusa.db.DBColumn;

/**
 * AUTOAREA レンデラは、カラムのデータをテキストエリアで表示する場合に
 * 使用するクラスです。
 * readonlyのテキストエリアでclass=renderer-textareaとして出力し、
 * name属性は付けません。（データは送信されません）
 * エリアの縦、横サイズはエディタの自動拡張テキストエリア(AUTOAREA)と同様にして算出されます。
 *
 * カラムの表示に必要な属性は, DBColumn オブジェクト より取り出します。
 * このクラスは、DBColumn オブジェクト毎に１つ作成されます。
 *
 * @og.rev 5.3.4.0 (2011/04/01) 新規作成
 * @og.group データ編集
 *
 * @version  4.0
 * @author   Hiroki Nakamura
 * @since    JDK5.0,
 */
public class Renderer_AUTOAREA extends AbstractRenderer {
	//* このプログラムのVERSION文字列を設定します。	{@value} */
	private static final String VERSION = "4.0.0 (2005/08/31)" ;

	private final int COLUMNS_MAXSIZE = HybsSystem.sysInt( "HTML_COLUMNS_MAXSIZE" );
	// viewタグで表示する場合のカラムの大きさ
	private final int VIEW_COLUMNS_MAXSIZE = HybsSystem.sysInt( "HTML_VIEW_COLUMNS_MAXSIZE" );

	private  String		size1;
	private  String		size2;

	private static final int COL = 0 ;
	private static final int ROW = 1 ;
	private static final String CODE = "Windows-31J";

	private	int	cols1	= 0 ;
	private	int	cols2	= 0 ;
	private	int	rows1	= 0 ;
	private	int	rows2	= 0 ;
	private int maxColSize = HybsSystem.sysInt( "HTML_AUTOAREA_MAX_COL_SIZE" );
	private int maxRowSize = HybsSystem.sysInt( "HTML_AUTOAREA_MAX_ROW_SIZE" );

	private final TagBuffer	tagBuffer = new TagBuffer();

	/**
	 * デフォルトコンストラクター。
	 * このコンストラクターで、基本オブジェクトを作成します。
	 *
	 */
	public Renderer_AUTOAREA() {
		// 何もありません。(super を呼び出しません)
	}

	/**
	 * コンストラクター
	 * textareaのサイズを決めるため、sizeとrowを決定する
	 * editorの計算を移植。
	 *
	 * @param   clm DBColumn オブジェクト
	 */
	private Renderer_AUTOAREA( final DBColumn clm ) {
		String size = clm.getViewLength();
		int maxlength = clm.getTotalSize();

		if( size != null ) {
			if( size.indexOf( ',' ) >= 0 ) {
				size = size.substring( 0, size.indexOf( ',' ) );
			}
			size1 = size;
			size2 = size;
		}
		else {
			size1 = String.valueOf( clm.getFieldSize( maxlength, COLUMNS_MAXSIZE ) );
			size2 = String.valueOf( clm.getFieldSize( maxlength, VIEW_COLUMNS_MAXSIZE ) );
		}
		
		int r1 = (clm.getTotalSize()/Integer.parseInt(size1)) + 1;	// 4.0.0 (2005/01/31) メソッド名変更
		if( r1 > 5 ) { rows1 = 5; }
		else { rows1 = r1; }

		int r2 = (clm.getTotalSize()/Integer.parseInt(size2)) + 1;	// 4.0.0 (2005/01/31) メソッド名変更
		if( r2 > 5 ) { rows2 = 5; }
		else { rows2 = r2; }

		// 3.8.0.2 (2005/07/11) size に、"rows-maxRow,cols-maxCols" を指定
		String param = StringUtil.nval( clm.getRendererParam(),clm.getViewLength() );
		if( param != null && param.length() != 0 ) {
			int prmAdrs = param.indexOf( ',' );
			if( prmAdrs > 0 ) {
				String rowStr = param.substring( 0,prmAdrs );
				String colStr = param.substring( prmAdrs+1 );

				int rowAdrs = rowStr.indexOf( '-' );	// rows-maxRow 設定時 '-' がなければ、ただのrows
				if( rowAdrs > 0 ) {
					rows1 = Integer.parseInt( rowStr.substring( 0,rowAdrs ) );
					maxRowSize = Integer.parseInt( rowStr.substring( rowAdrs+1 ) );
				}
				else {
					rows1 = Integer.parseInt( rowStr );
				}
				rows2 = rows1 ;

				int colAdrs = colStr.indexOf( '-' );	// cols-maxCols 設定時 '-' がなければ、ただのcols
				if( colAdrs > 0 ) {
					cols1 = Integer.parseInt( colStr.substring( 0,colAdrs ) );
					maxColSize = Integer.parseInt( colStr.substring( colAdrs+1 ) );
				}
				else {
					cols1 = Integer.parseInt( colStr );
				}
				cols2 = cols1;
			}
		}
	}

	/**
	 * 各オブジェクトから自分のインスタンスを返します。
	 * 自分自身をキャッシュするのか、新たに作成するのかは、各サブクラスの実装に
	 * まかされます。
	 *
	 * @param   clm DBColumn オブジェクト
	 * @return  CellRenderer オブジェクト
	 */
	public CellRenderer newInstance( final DBColumn clm ) {
		return new Renderer_AUTOAREA( clm );
	}

	/**
	 * データの表示用文字列を返します。
	 *
	 * @param   value String
	 * @return  データの表示用文字列
	 */
	public String getValue( final String value ) {

		int[] rowcol = getRowsCols( value,cols1,rows1 );

		TagBuffer tag = new TagBuffer( "textarea" );
		tag.add( "cols"    , String.valueOf( rowcol[COL] ) );
		tag.add( "rows"    , String.valueOf( rowcol[ROW] ) );
		tag.add( "readonly", "readonly" );
		tag.add( "class"   , "renderer-textarea" );
		tag.add( tagBuffer.makeTag() );
		tag.setBody( value );

		return tag.makeTag();
	}

	/**
	 * データの表示用文字列を返します。
	 *
	 * @param   row   int 行番号
	 * @param   value String
	 * @return  データ表示用の文字列
	 */
	public String getValue( final int row,final String value ) {

		int[] rowcol = getRowsCols( value,cols2,rows2 );

		TagBuffer tag = new TagBuffer( "textarea" );
		tag.add( "cols"    , String.valueOf( rowcol[COL] ) );
		tag.add( "rows"    , String.valueOf( rowcol[ROW] ) );
		tag.add( "readonly", "readonly" );
		tag.add( "class"   , "renderer-textarea" );
		tag.add( tagBuffer.makeTag() );
		tag.setBody( value );

		return tag.makeTag( row,value );
	}

	/**
	 * 自動表示する行列の数を求めます。
	 * 行数は、引数の文字列中に含まれる 改行コードの個数を求めます。
	 * 列数は、各行数のなかの最大桁数より求めます。これには半角、全角が含まれる為、
	 * 半角換算での文字数ということになります。
	 * 行数と列数が、初期設定の行数と列数より小さい場合は、初期設定値が使用されます。
	 *
	 * @param   value String 表示文字列
	 * @param   cols int 最小カラム数
	 * @param   rows int 最小行数
	 * @return  int[] 自動計算した行列の配列
	 */
	private int[] getRowsCols( final String value,final int cols, final int rows ) {
		if( value == null ) {
			return new int[] { rows,cols };
		}

		StringTokenizer token = new StringTokenizer( value, "\n", true );

		int cntRow = 1;
		int maxCol = 0;
		while ( token.hasMoreTokens() ) {
			String val = token.nextToken();
			if( "\n".equals( val ) ) { cntRow++; }
			else {
				byte[] byteValue = StringUtil.makeByte( val,CODE );	// 3.5.5.3 (2004/04/09)
				int byteSize = byteValue.length;
				if( maxColSize > 0 && byteSize > maxColSize ) {		// 最大列数
					cntRow += (byteSize / maxColSize);
					maxCol = maxColSize ;
				}
				else if( byteSize > maxCol ) { maxCol = byteSize; }
			}
			if( maxRowSize > 0 && cntRow >= maxRowSize ) {		// 最大行数
				cntRow = maxRowSize;
				break;
			}
		}

		maxCol += 2;	// マージン。フォントや画面サイズに影響する為、比率のほうがよい？

		int[] rtn = new int[2];
		rtn[ROW] = (rows<cntRow) ? cntRow : rows ;
		rtn[COL] = (cols<maxCol) ? maxCol : cols ;

		return rtn ;
	}
}

==> openGion_dev_5.5.5.0.zip: opengion/uap/webapps/gf/src/org/opengion/plugin/column/Renderer_AUTOAREA.java <==
/*
 * Copyright (c) 2009 The openGion Project.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.opengion.plugin.column;

import java.util.StringTokenizer;

import org.opengion.fukurou.util.StringUtil;
import org.opengion.fukurou.util.TagBuffer;
import org.opengion.hayabusa.common.HybsSystem;
import org.opengion.hayabusa.db.AbstractRenderer;
import org.opengion.hayabusa.db.CellRenderer;
import org.opengion.hayabusa.db.DBColumn;

/**
 * AUTOAREA レンデラは、カラムのデータをテキストエリアで表示する場合に
 * 使用するクラスです。
 * readonlyのテキストエリアでclass=renderer-textareaとして出力し、
 * name属性は付けません。(データは送信されません)
 * エリアの縦、横サイズはエディタの自動拡張テキストエリア(AUTOAREA)と同様にして算出されます。
 *
 * カラムの表示に必要な属性は, DBColumn オブジェクト より取り出します。
 * このクラスは、DBColumn オブジェクト毎に１つ作成されます。
 *
 * @og.rev 5.3.4.0 (2011/04/01) 新規作成
 * @og.group データ編集
 *
 * @version  4.0
 * @author   Hiroki Nakamura
 * @since    JDK5.0,
 */
public class Renderer_AUTOAREA extends AbstractRenderer {
	//* このプログラムのVERSION文字列を設定します。	{@value} */
	private static final String VERSION = "4.0.0 (2005/08/31)" ;

	private final int COLUMNS_MAXSIZE = HybsSystem.sysInt( "HTML_COLUMNS_MAXSIZE" );
	// viewタグで表示する場合のカラムの大きさ
	private final int VIEW_COLUMNS_MAXSIZE = HybsSystem.sysInt( "HTML_VIEW_COLUMNS_MAXSIZE" );

	private  String		size1;
	private  String		size2;

	private static final int COL = 0 ;
	private static final int ROW = 1 ;
	private static final String CODE = "Windows-31J";

	private	int	cols1	= 0 ;
	private	int	cols2	= 0 ;
	private	int	rows1	= 0 ;
	private	int	rows2	= 0 ;
	private int maxColSize = HybsSystem.sysInt( "HTML_AUTOAREA_MAX_COL_SIZE" );
	private int maxRowSize = HybsSystem.sysInt( "HTML_AUTOAREA_MAX_ROW_SIZE" );

	private final TagBuffer	tagBuffer = new TagBuffer();

	/**
	 * デフォルトコンストラクター。
	 * このコンストラクターで、基本オブジェクトを作成します。
	 *
	 */
	public Renderer_AUTOAREA() {
		// 何もありません。(super を呼び出しません)
	}

	/**
	 * コンストラクター
	 * textareaのサイズを決めるため、sizeとrowを決定する
	 * editorの計算を移植。
	 *
	 * @param	clm	DBColumnオブジェクト
	 */
	private Renderer_AUTOAREA( final DBColumn clm ) {
		String size = clm.getViewLength();
		int maxlength = clm.getTotalSize();

		if( size != null ) {
			if( size.indexOf( ',' ) >= 0 ) {
				size = size.substring( 0, size.indexOf( ',' ) );
			}
			size1 = size;
			size2 = size;
		}
		else {
			size1 = String.valueOf( clm.getFieldSize( maxlength, COLUMNS_MAXSIZE ) );
			size2 = String.valueOf( clm.getFieldSize( maxlength, VIEW_COLUMNS_MAXSIZE ) );
		}

		int r1 = (clm.getTotalSize()/Integer.parseInt(size1)) + 1;	// 4.0.0 (2005/01/31) メソッド名変更
		if( r1 > 5 ) { rows1 = 5; }
		else { rows1 = r1; }

		int r2 = (clm.getTotalSize()/Integer.parseInt(size2)) + 1;	// 4.0.0 (2005/01/31) メソッド名変更
		if( r2 > 5 ) { rows2 = 5; }
		else { rows2 = r2; }

		// 3.8.0.2 (2005/07/11) size に、"rows-maxRow,cols-maxCols" を指定
		String param = StringUtil.nval( clm.getRendererParam(),clm.getViewLength() );
		if( param != null && param.length() != 0 ) {
			int prmAdrs = param.indexOf( ',' );
			if( prmAdrs > 0 ) {
				String rowStr = param.substring( 0,prmAdrs );
				String colStr = param.substring( prmAdrs+1 );

				int rowAdrs = rowStr.indexOf( '-' );	// rows-maxRow 設定時 '-' がなければ、ただのrows
				if( rowAdrs > 0 ) {
					rows1 = Integer.parseInt( rowStr.substring( 0,rowAdrs ) );
					maxRowSize = Integer.parseInt( rowStr.substring( rowAdrs+1 ) );
				}
				else {
					rows1 = Integer.parseInt( rowStr );
				}
				rows2 = rows1 ;

				int colAdrs = colStr.indexOf( '-' );	// cols-maxCols 設定時 '-' がなければ、ただのcols
				if( colAdrs > 0 ) {
					cols1 = Integer.parseInt( colStr.substring( 0,colAdrs ) );
					maxColSize = Integer.parseInt( colStr.substring( colAdrs+1 ) );
				}
				else {
					cols1 = Integer.parseInt( colStr );
				}
				cols2 = cols1;
			}
		}
	}

	/**
	 * 各オブジェクトから自分のインスタンスを返します。
	 * 自分自身をキャッシュするのか、新たに作成するのかは、各サブクラスの実装に
	 * まかされます。
	 *
	 * @param	clm	DBColumnオブジェクト
	 *
	 * @return	CellRendererオブジェクト
	 */
	public CellRenderer newInstance( final DBColumn clm ) {
		return new Renderer_AUTOAREA( clm );
	}

	/**
	 * データの表示用文字列を返します。
	 *
	 * @param   value 入力値
	 *
	 * @return  データの表示用文字列
	 */
	@Override
	public String getValue( final String value ) {

		int[] rowcol = getRowsCols( value,cols1,rows1 );

		TagBuffer tag = new TagBuffer( "textarea" );
		tag.add( "cols"    , String.valueOf( rowcol[COL] ) );
		tag.add( "rows"    , String.valueOf( rowcol[ROW] ) );
		tag.add( "readonly", "readonly" );
		tag.add( "class"   , "renderer-textarea" );
		tag.add( tagBuffer.makeTag() );
		tag.setBody( value );

		return tag.makeTag();
	}

	/**
	 * データの表示用文字列を返します。
	 *
	 * @param   row   行番号
	 * @param   value 入力値
	 *
	 * @return  データ表示用の文字列
	 */
	@Override
	public String getValue( final int row,final String value ) {

		int[] rowcol = getRowsCols( value,cols2,rows2 );

		TagBuffer tag = new TagBuffer( "textarea" );
		tag.add( "cols"    , String.valueOf( rowcol[COL] ) );
		tag.add( "rows"    , String.valueOf( rowcol[ROW] ) );
		tag.add( "readonly", "readonly" );
		tag.add( "class"   , "renderer-textarea" );
		tag.add( tagBuffer.makeTag() );
		tag.setBody( value );

		return tag.makeTag( row,value );
	}

	/**
	 * 自動表示する行列の数を求めます。
	 * 行数は、引数の文字列中に含まれる 改行コードの個数を求めます。
	 * 列数は、各行数のなかの最大桁数より求めます。これには半角、全角が含まれる為、
	 * 半角換算での文字数ということになります。
	 * 行数と列数が、初期設定の行数と列数より小さい場合は、初期設定値が使用されます。
	 *
	 * @param   value 入力値 表示文字列
	 * @param	cols	最小カラム数
	 * @param	rows	最小行数
	 *
	 * @return	自動計算した行列の配列
	 */
	private int[] getRowsCols( final String value,final int cols, final int rows ) {
		if( value == null ) {
			return new int[] { rows,cols };
		}

		StringTokenizer token = new StringTokenizer( value, "\n", true );

		int cntRow = 1;
		int maxCol = 0;
		while ( token.hasMoreTokens() ) {
			String val = token.nextToken();
			if( "\n".equals( val ) ) { cntRow++; }
			else {
				byte[] byteValue = StringUtil.makeByte( val,CODE );	// 3.5.5.3 (2004/04/09)
				int byteSize = byteValue.length;
				if( maxColSize > 0 && byteSize > maxColSize ) {		// 最大列数
					cntRow += (byteSize / maxColSize);
					maxCol = maxColSize ;
				}
				else if( byteSize > maxCol ) { maxCol = byteSize; }
			}
			if( maxRowSize > 0 && cntRow >= maxRowSize ) {		// 最大行数
				cntRow = maxRowSize;
				break;
			}
		}

		maxCol += 2;	// マージン。フォントや画面サイズに影響する為、比率のほうがよい？

		int[] rtn = new int[2];
		rtn[ROW] = (rows<cntRow) ? cntRow : rows ;
		rtn[COL] = (cols<maxCol) ? maxCol : cols ;

		return rtn ;
	}
}
